"""
Python module generated from Java source file org.bukkit.material.Comparator

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Material
from org.bukkit.block import BlockFace
from org.bukkit.material import *
from typing import Any, Callable, Iterable, Tuple


class Comparator(MaterialData, Directional, Redstone):
    """
    Represents a comparator in the on or off state, in normal or subtraction mode and facing in a specific direction.

    See
    - Material.LEGACY_REDSTONE_COMPARATOR_ON

    Deprecated
    - all usage of MaterialData is deprecated and subject to removal.
    Use org.bukkit.block.data.BlockData.
    """

    def __init__(self):
        """
        Constructs a comparator switched off, with the default mode (normal) and facing the default direction (north).
        """
        ...


    def __init__(self, facingDirection: "BlockFace"):
        """
        Constructs a comparator switched off, with the default mode (normal) and facing the specified direction.

        Arguments
        - facingDirection: the direction the comparator is facing

        See
        - BlockFace
        """
        ...


    def __init__(self, facingDirection: "BlockFace", isSubtraction: bool):
        """
        Constructs a comparator switched off, with the specified mode and facing the specified direction.

        Arguments
        - facingDirection: the direction the comparator is facing
        - isSubtraction: True if the comparator is in subtraction mode, False for normal comparator operation

        See
        - BlockFace
        """
        ...


    def __init__(self, facingDirection: "BlockFace", isSubtraction: bool, state: bool):
        """
        Constructs a comparator switched on or off, with the specified mode and facing the specified direction.

        Arguments
        - facingDirection: the direction the comparator is facing
        - isSubtraction: True if the comparator is in subtraction mode, False for normal comparator operation
        - state: True if the comparator is in the on state

        See
        - BlockFace
        """
        ...


    def __init__(self, type: "Material"):
        ...


    def __init__(self, type: "Material", data: int):
        """
        Arguments
        - type: the type
        - data: the raw data value

        Deprecated
        - Magic value
        """
        ...


    def setSubtractionMode(self, isSubtraction: bool) -> None:
        """
        Sets whether the comparator is in subtraction mode.

        Arguments
        - isSubtraction: True if the comparator is in subtraction mode, False for normal comparator operation
        """
        ...


    def isSubtractionMode(self) -> bool:
        """
        Checks whether the comparator is in subtraction mode

        Returns
        - True if the comparator is in subtraction mode, False if normal comparator operation
        """
        ...


    def setFacingDirection(self, face: "BlockFace") -> None:
        """
        Sets the direction this comparator is facing

        Arguments
        - face: The direction to set this comparator to

        See
        - BlockFace
        """
        ...


    def getFacing(self) -> "BlockFace":
        """
        Gets the direction this comparator is facing

        Returns
        - The direction this comparator is facing

        See
        - BlockFace
        """
        ...


    def toString(self) -> str:
        ...


    def clone(self) -> "Comparator":
        ...


    def isPowered(self) -> bool:
        """
        Checks if the comparator is powered

        Returns
        - True if the comparator is powered
        """
        ...


    def isBeingPowered(self) -> bool:
        """
        Checks if the comparator is being powered

        Returns
        - True if the comparator is being powered
        """
        ...



"""
Python module generated from Java source file com.google.common.collect.ImmutableMapEntry

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.collect import *
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class ImmutableMapEntry(ImmutableEntry):
    """
    Implementation of `Entry` for ImmutableMap that adds extra methods to traverse hash
    buckets for the key and the value. This allows reuse in RegularImmutableMap and RegularImmutableBiMap, which don't have to recopy the entries created by their `Builder`
    implementations.
    
    This base implementation has no key or value pointers, so instances of ImmutableMapEntry (but
    not its subclasses) can be reused when copied from one ImmutableMap to another.

    Author(s)
    - Louis Wasserman
    """





"""
Python module generated from Java source file com.google.common.cache.CacheLoader

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.base import Function
from com.google.common.base import Supplier
from com.google.common.cache import *
from com.google.common.util.concurrent import Futures
from com.google.common.util.concurrent import ListenableFuture
from com.google.common.util.concurrent import ListenableFutureTask
from java.io import Serializable
from java.util.concurrent import Executor
from typing import Any, Callable, Iterable, Tuple


class CacheLoader:
    """
    Computes or retrieves values, based on a key, for use in populating a LoadingCache.
    
    Most implementations will only need to implement .load. Other methods may be
    overridden as desired.
    
    Usage example:
    
    ````CacheLoader<Key, Graph> loader = new CacheLoader<Key, Graph>() {
      public Graph load(Key key) throws AnyException {
        return createExpensiveGraph(key);`
    };
    LoadingCache<Key, Graph> cache = CacheBuilder.newBuilder().build(loader);
    }```
    
    Since this example doesn't support reloading or bulk loading, it can also be specified much
    more simply:
    
    ````CacheLoader<Key, Graph> loader = CacheLoader.from(key -> createExpensiveGraph(key));````

    Author(s)
    - Charles Fry

    Since
    - 10.0
    """

    def load(self, key: "K") -> "V":
        """
        Computes or retrieves the value corresponding to `key`.

        Arguments
        - key: the non-null key whose value should be loaded

        Returns
        - the value associated with `key`; **must not be null**

        Raises
        - Exception: if unable to load the result
        - InterruptedException: if this method is interrupted. `InterruptedException` is
            treated like any other `Exception` in all respects except that, when it is caught,
            the thread's interrupt status is set
        """
        ...


    def reload(self, key: "K", oldValue: "V") -> "ListenableFuture"["V"]:
        """
        Computes or retrieves a replacement value corresponding to an already-cached `key`. This
        method is called when an existing cache entry is refreshed by CacheBuilder.refreshAfterWrite, or through a call to LoadingCache.refresh.
        
        This implementation synchronously delegates to .load. It is recommended that it be
        overridden with an asynchronous implementation when using CacheBuilder.refreshAfterWrite.
        
        **Note:** *all exceptions thrown by this method will be logged and then swallowed*.

        Arguments
        - key: the non-null key whose value should be loaded
        - oldValue: the non-null old value corresponding to `key`

        Returns
        - the future new value associated with `key`; **must not be null, must not return
            null**

        Raises
        - Exception: if unable to reload the result
        - InterruptedException: if this method is interrupted. `InterruptedException` is
            treated like any other `Exception` in all respects except that, when it is caught,
            the thread's interrupt status is set

        Since
        - 11.0
        """
        ...


    def loadAll(self, keys: Iterable["K"]) -> dict["K", "V"]:
        """
        Computes or retrieves the values corresponding to `keys`. This method is called by LoadingCache.getAll.
        
        If the returned map doesn't contain all requested `keys` then the entries it does
        contain will be cached, but `getAll` will throw an exception. If the returned map
        contains extra keys not present in `keys` then all returned entries will be cached, but
        only the entries for `keys` will be returned from `getAll`.
        
        This method should be overridden when bulk retrieval is significantly more efficient than
        many individual lookups. Note that LoadingCache.getAll will defer to individual calls
        to LoadingCache.get if this method is not overridden.

        Arguments
        - keys: the unique, non-null keys whose values should be loaded

        Returns
        - a map from each key in `keys` to the value associated with that key; **may not
            contain null values**

        Raises
        - Exception: if unable to load the result
        - InterruptedException: if this method is interrupted. `InterruptedException` is
            treated like any other `Exception` in all respects except that, when it is caught,
            the thread's interrupt status is set

        Since
        - 11.0
        """
        ...


    @staticmethod
    def from(function: "Function"["K", "V"]) -> "CacheLoader"["K", "V"]:
        """
        Returns a cache loader that uses `function` to load keys, without supporting either
        reloading or bulk loading. This allows creating a cache loader using a lambda expression.
        
        The returned object is serializable if `function` is serializable.

        Arguments
        - function: the function to be used for loading values; must never return `null`

        Returns
        - a cache loader that loads values by passing each key to `function`
        """
        ...


    @staticmethod
    def from(supplier: "Supplier"["V"]) -> "CacheLoader"["Object", "V"]:
        """
        Returns a cache loader based on an *existing* supplier instance. Note that there's no need
        to create a *new* supplier just to pass it in here; just subclass `CacheLoader` and
        implement .load load instead.
        
        The returned object is serializable if `supplier` is serializable.

        Arguments
        - supplier: the supplier to be used for loading values; must never return `null`

        Returns
        - a cache loader that loads values by calling Supplier.get, irrespective of the
            key
        """
        ...


    @staticmethod
    def asyncReloading(loader: "CacheLoader"["K", "V"], executor: "Executor") -> "CacheLoader"["K", "V"]:
        """
        Returns a `CacheLoader` which wraps `loader`, executing calls to CacheLoader.reload using `executor`.
        
        This method is useful only when `loader.reload` has a synchronous implementation, such
        as .reload the default implementation.

        Since
        - 17.0
        """
        ...


    class UnsupportedLoadingOperationException(UnsupportedOperationException):
        """
        Exception thrown by `loadAll()` to indicate that it is not supported.

        Since
        - 19.0
        """




    class InvalidCacheLoadException(RuntimeException):
        """
        Thrown to indicate that an invalid response was returned from a call to CacheLoader.

        Since
        - 11.0
        """

        def __init__(self, message: str):
            ...



"""
Python module generated from Java source file com.google.common.net.HostSpecifier

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.base import Preconditions
from com.google.common.net import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.net import InetAddress
from java.text import ParseException
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class HostSpecifier:
    """
    A syntactically valid host specifier, suitable for use in a URI. This may be either a numeric IP
    address in IPv4 or IPv6 notation, or a domain name.
    
    Because this class is intended to represent host specifiers which can reasonably be used in a
    URI, the domain name case is further restricted to include only those domain names which end in a
    recognized public suffix; see InternetDomainName.isPublicSuffix() for details.
    
    Note that no network lookups are performed by any `HostSpecifier` methods. No attempt is
    made to verify that a provided specifier corresponds to a real or accessible host. Only syntactic
    and pattern-based checks are performed.
    
    If you know that a given string represents a numeric IP address, use InetAddresses to
    obtain and manipulate a java.net.InetAddress instance from it rather than using this
    class. Similarly, if you know that a given string represents a domain name, use InternetDomainName rather than this class.

    Author(s)
    - Craig Berry

    Since
    - 5.0
    """

    @staticmethod
    def fromValid(specifier: str) -> "HostSpecifier":
        """
        Returns a `HostSpecifier` built from the provided `specifier`, which is already
        known to be valid. If the `specifier` might be invalid, use .from(String)
        instead.
        
        The specifier must be in one of these formats:
        
        
          - A domain name, like `google.com`
          - A IPv4 address string, like `127.0.0.1`
          - An IPv6 address string with or without brackets, like `[2001:db8::1]` or `2001:db8::1`

        Raises
        - IllegalArgumentException: if the specifier is not valid.
        """
        ...


    @staticmethod
    def from(specifier: str) -> "HostSpecifier":
        """
        Attempts to return a `HostSpecifier` for the given string, throwing an exception if
        parsing fails. Always use this method in preference to .fromValid(String) for a
        specifier that is not already known to be valid.

        Raises
        - ParseException: if the specifier is not valid.
        """
        ...


    @staticmethod
    def isValid(specifier: str) -> bool:
        """
        Determines whether `specifier` represents a valid HostSpecifier as described in
        the documentation for .fromValid(String).
        """
        ...


    def equals(self, other: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...


    def toString(self) -> str:
        """
        Returns a string representation of the host specifier suitable for inclusion in a URI. If the
        host specifier is a domain name, the string will be normalized to all lower case. If the
        specifier was an IPv6 address without brackets, brackets are added so that the result will be
        usable in the host part of a URI.
        """
        ...



"""
Python module generated from Java source file org.bukkit.material.Wood

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Material
from org.bukkit import TreeSpecies
from org.bukkit.material import *
from typing import Any, Callable, Iterable, Tuple


class Wood(MaterialData):
    """
    Represents wood blocks of different species.

    See
    - Material.LEGACY_WOOD_DOUBLE_STEP

    Deprecated
    - all usage of MaterialData is deprecated and subject to removal.
    Use org.bukkit.block.data.BlockData.
    """

    def __init__(self):
        """
        Constructs a wood block.
        """
        ...


    def __init__(self, species: "TreeSpecies"):
        """
        Constructs a wood block of the given tree species.

        Arguments
        - species: the species of the wood block
        """
        ...


    def __init__(self, type: "Material"):
        """
        Constructs a wood block of the given type.

        Arguments
        - type: the type of wood block
        """
        ...


    def __init__(self, type: "Material", species: "TreeSpecies"):
        """
        Constructs a wood block of the given type and tree species.

        Arguments
        - type: the type of wood block
        - species: the species of the wood block
        """
        ...


    def __init__(self, type: "Material", data: int):
        """
        Arguments
        - type: the type
        - data: the raw data value

        Deprecated
        - Magic value
        """
        ...


    def getSpecies(self) -> "TreeSpecies":
        """
        Gets the current species of this wood block

        Returns
        - TreeSpecies of this wood block
        """
        ...


    def setSpecies(self, species: "TreeSpecies") -> None:
        """
        Sets the species of this wood block

        Arguments
        - species: New species of this wood block
        """
        ...


    def toString(self) -> str:
        ...


    def clone(self) -> "Wood":
        ...



"""
Python module generated from Java source file org.bukkit.entity.Goat

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class Goat(Animals):
    """
    A Goat.
    """

    def hasLeftHorn(self) -> bool:
        """
        Gets if this goat has its left horn.

        Returns
        - left horn status
        """
        ...


    def setLeftHorn(self, hasHorn: bool) -> None:
        """
        Sets if this goat has its left horn.

        Arguments
        - hasHorn: left horn status
        """
        ...


    def hasRightHorn(self) -> bool:
        """
        Gets if this goat has its right horn.

        Returns
        - right horn status
        """
        ...


    def setRightHorn(self, hasHorn: bool) -> None:
        """
        Sets if this goat has its right horn.

        Arguments
        - hasHorn: right horn status
        """
        ...


    def isScreaming(self) -> bool:
        """
        Gets if this is a screaming goat.
        
        A screaming goat makes screaming sounds and rams more often. They do not
        offer home loans.

        Returns
        - screaming status
        """
        ...


    def setScreaming(self, screaming: bool) -> None:
        """
        Sets if this is a screaming goat.
        
        A screaming goat makes screaming sounds and rams more often. They do not
        offer home loans.

        Arguments
        - screaming: screaming status
        """
        ...



from .DamageType import DamageType
from .DeathMessageType import DeathMessageType
from .DamageScaling import DamageScaling
from .DamageSource import DamageSource
from .DamageEffect import DamageEffect


"""
Python module generated from Java source file com.google.common.math.LinearTransformation

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.math import *
from com.google.errorprone.annotations.concurrent import LazyInit
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class LinearTransformation:
    """
    The representation of a linear transformation between real numbers `x` and `y`.
    Graphically, this is the specification of a straight line on a plane. The transformation can be
    expressed as `y = m * x + c` for finite `m` and `c`, unless it is a vertical
    transformation in which case `x` has a constant value for all `y`. In the
    non-vertical case, `m` is the slope of the transformation (and a horizontal transformation
    has zero slope).

    Author(s)
    - Pete Gillin

    Since
    - 20.0
    """

    @staticmethod
    def mapping(x1: float, y1: float) -> "LinearTransformationBuilder":
        """
        Start building an instance which maps `x = x1` to `y = y1`. Both arguments must be
        finite. Call either LinearTransformationBuilder.and or LinearTransformationBuilder.withSlope on the returned object to finish building the instance.
        """
        ...


    @staticmethod
    def vertical(x: float) -> "LinearTransformation":
        """
        Builds an instance representing a vertical transformation with a constant value of `x`.
        (The inverse of this will be a horizontal transformation.)
        """
        ...


    @staticmethod
    def horizontal(y: float) -> "LinearTransformation":
        """
        Builds an instance representing a horizontal transformation with a constant value of `y`.
        (The inverse of this will be a vertical transformation.)
        """
        ...


    @staticmethod
    def forNaN() -> "LinearTransformation":
        """
        Builds an instance for datasets which contains Double.NaN. The .isHorizontal
        and .isVertical methods return `False` and the .slope, and .transform methods all return Double.NaN. The .inverse method returns the same
        instance.
        """
        ...


    def isVertical(self) -> bool:
        """
        Returns whether this is a vertical transformation.
        """
        ...


    def isHorizontal(self) -> bool:
        """
        Returns whether this is a horizontal transformation.
        """
        ...


    def slope(self) -> float:
        """
        Returns the slope of the transformation, i.e. the rate of change of `y` with respect to
        `x`. This must not be called on a vertical transformation (i.e. when .isVertical() is True).
        """
        ...


    def transform(self, x: float) -> float:
        """
        Returns the `y` corresponding to the given `x`. This must not be called on a
        vertical transformation (i.e. when .isVertical() is True).
        """
        ...


    def inverse(self) -> "LinearTransformation":
        """
        Returns the inverse linear transformation. The inverse of a horizontal transformation is a
        vertical transformation, and vice versa. The inverse of the .forNaN transformation is
        itself. In all other cases, the inverse is a transformation such that applying both the
        original transformation and its inverse to a value gives you the original value give-or-take
        numerical errors. Calling this method multiple times on the same instance will always return
        the same instance. Calling this method on the result of calling this method on an instance will
        always return that original instance.
        """
        ...


    class LinearTransformationBuilder:
        """
        This is an intermediate stage in the construction process. It is returned by LinearTransformation.mapping. You almost certainly don't want to keep instances around, but
        instead use method chaining. This represents a single point mapping, i.e. a mapping between one
        `x` and `y` value pair.

        Since
        - 20.0
        """

        def and(self, x2: float, y2: float) -> "LinearTransformation":
            """
            Finish building an instance which also maps `x = x2` to `y = y2`. These values
            must not both be identical to the values given in the first mapping. If only the `x`
            values are identical, the transformation is vertical. If only the `y` values are
            identical, the transformation is horizontal (i.e. the slope is zero).
            """
            ...


        def withSlope(self, slope: float) -> "LinearTransformation":
            """
            Finish building an instance with the given slope, i.e. the rate of change of `y` with
            respect to `x`. The slope must not be `NaN`. It may be infinite, in which case
            the transformation is vertical. (If it is zero, the transformation is horizontal.)
            """
            ...
